# Generated by Django 5.2.5 on 2025-09-03 17:21

import django.db.models.deletion
import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("whitebox_plugin_flight_management", "0001_initial"),
    ]

    operations = [
        migrations.CreateModel(
            name="FlightSessionRecording",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("created_at", models.DateTimeField(default=django.utils.timezone.now)),
                ("file", models.FileField(upload_to="")),
                (
                    "flight_session",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="recordings",
                        to="whitebox_plugin_flight_management.flightsession",
                    ),
                ),
            ],
        ),
    ]
